/* *INDENT-OFF* */ /* THIS FILE IS GENERATED */

/* A register protocol for GDB, the GNU debugger.
   Copyright (C) 2001-2013 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* This file was created with the aid of ``regdat.sh'' and ``../../gdb-14.2/gdbserver/../gdb/regformats/microblaze-linux.dat''.  */

#include "server.h"
#include "regdef.h"
#include "tdesc.h"

const struct target_desc *tdesc_microblaze_linux;

void init_registers_microblaze_linux (void);
void
init_registers_microblaze_linux (void)
{
  static struct target_desc tdesc_microblaze_linux_s;
  struct target_desc *result = &tdesc_microblaze_linux_s;
  struct tdesc_feature *feature = tdesc_create_feature (result, "microblaze_linux");
  tdesc_create_reg (feature, "r0",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r1",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r2",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r3",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r4",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r5",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r6",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r7",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r8",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r9",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r10",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r11",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r12",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r13",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r14",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r15",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r16",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r17",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r18",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r19",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r20",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r21",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r22",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r23",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r24",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r25",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r26",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r27",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r28",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r29",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r30",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "r31",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpc",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rmsr",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rear",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "resr",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rfsr",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rbtr",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpvr0",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpvr1",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpvr2",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpvr3",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpvr4",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpvr5",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpvr6",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpvr7",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpvr8",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpvr9",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpvr10",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpvr11",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "redr",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rpid",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rzpr",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rtlbx",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rtlbsx",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rtlblo",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "rtlbhi",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "slr",
  0, 0, NULL, 32, NULL);
  tdesc_create_reg (feature, "shr",
  0, 0, NULL, 32, NULL);

static const char *expedite_regs_microblaze_linux[] = { "r1", "rpc", 0 };
#ifndef IN_PROCESS_AGENT
static const char *xmltarget_microblaze_linux = "microblaze-linux.xml";

  result->xmltarget = xmltarget_microblaze_linux;
#endif

  init_target_desc (result, expedite_regs_microblaze_linux);

  tdesc_microblaze_linux = result;
}
